;Copywrite 2000-2003 Samuel Stearley  sstear70@calvin.edu





;VERSION HISTORY OF THIS FILE:
;________________________________________________________
;v1	-First release

;v1.1	-Second release, I found out that a68k was not
;	 optomizing certain brancing routines to the
;	 short form so I force it with the .s suffix.
;	 This is a bug in a68k that has since been fixed.
;

;v1.2	-Third Release, mentioned how the buffer will never
;	 give out before the RLE run is over.

;v1.3 -Fourth Release, added a vertical rle decompression
;	 routine for use with the latest istudio version.



;Rle stands for "Run Length Encoding"  Imagine that an entire line
;of a picture you want to copy to the video memory is clear.
;So that line is really 30 zero bytes in a row.  With rle that
;will condense into: A) a tag.  B) the value to repeat. And
;C) the number of times to repeat it.  That is 3 bytes instead
;of 30.  So you really only get good compression with rle if
;there is a lot of white space on the screen.

;I based it on the z80 rle decompression routine by David Phillips
;that I found at tcpa.calc.org.  Feel free to use it, but please
;thank me if you do.

;A RLE decompression routine written in C can be found in the
;bg_ source files of tichess.  The code below is 32 bytes and
;I do not think that the C equivilent will be as small or as
;fast.  But because the algorthym is very simple the speed
;differences will not be noticed.


;---------------------------------------------------
;Inputs:
;	d0- Size of buffer to decompress to minus one.
;	a0- Pointer to the rle compressed data
;	a1- Pointer to the buffer to decompress to.
;Destroys:
;	d0,d1,d2,a0,a1
;---------------------------------------------------
rle_loop:
	move.b	(a0)+,d1		;get next byte
	cmp.b		#$91,d1		;is it a run?
	beq.s		decode_run		;then decode the run
	move.b	d1,(a1)+		;else store it to memory
continue_rle:
	dbra		d0,rle_loop
	rts
decode_run:
	move.b	(a0)+,d1		;get byte to repeat
	move.b	(a0)+,d2		;get how many time to repeat
continue_run:
	move.b	d1,(a1)+		;move it
	subq.b	#1,d2			;decrease run variable
	dbeq		d0,continue_run	;go until the run is done or number of bytes is done
	bra.s		continue_rle	;and so we go on to the next byte


;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------


;This is an rle decompression routine that will decompress
;straight to the video memory.  It takes into acount the
;fact that with the ti89 you have to skip 10 bytes to get
;to the next line of video memory.  Just give it a pointer
;in a0 to the rle compressed picture.  The compressed picture
;should be 160 by 100 pixels.  It is optomized for size, not
;speed so the "check_line_end" function is not inlined.
;The rle decompression routine in the tichess code for the
;89 also takes into account the 10 bytes that needs to be
;skipped.

;there is no need to use this on a ti92+/v200


;It will destroy:  d0,d1,d2,d3,a0,a1



	lea		LCD_MEM,a1
	moveq		#20,d3

;----decompress picture to video memory----

	move.w	#1999,d0		;decompress 2000 bytes
rle_loop:
	move.b	(a0)+,d1		;get next byte	
	cmp.b		#$91,d1		;is it a run?
	beq.s		decode_run		;then decode the run
	move.b	d1,(a1)+		;else store it to video memory
	bsr.s		check_line_end
continue_rle:
	dbra		d0,rle_loop
	rts
decode_run:
	move.b	(a0)+,d1		;get byte to repeat
	move.b	(a0)+,d2		;get how many time to repeat
continue_run:
	move.b	d1,(a1)+		;move it
	bsr.s		check_line_end
	subq.b	#1,d2
	dbeq		d0,continue_run	;go until the run is done or number of bytes is done
	bra.s		continue_rle


check_line_end:
	subq.w	#1,d3
	bne.s		stay_on_line
	moveq		#20,d3
	lea		10(a1),a1
stay_on_line:
	rts


;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------



;---Inputs-------------------------------------------------------
;a0.l source data
;a1.l destination, for example this can be the video memory
;d0.w	number of bytes to write -1
;d1.l width of the buffer in bytes, use 30 if decompressing to
;	straight to video ram.
;d2.w number of line high, use 100 if decompressing straight to
;	the ti-89s screen.  Use 128 if the 92+ screen
;-----------------------------------------------------------------


DecompRLEVert:
	lea		1(a1),a2		;the next column of bytes, for when we have written this entire column
	move.w	d2,d3			;a temp counter

NormalBytes:
	move.b	(a0)+,d4		;get byte
	cmp.b		#$91,d4		;is it a run
	beq.s		DecodeRleRun
	move.b	d4,(a1)		;store byte
	add.l		d1,a1			;go down the column
	subq.w	#1,d3			;decrease the column counter
	bne.s		ContinueThisColumn1
	move.w	d2,d3			;new counter
	move.l	a2,a1			;start over at a new column
	addq.l	#1,a2			;what our next column will be
ContinueThisColumn1
	dbra		d0,NormalBytes
	rts
DecodeRleRun:
	move.b	(a0)+,d4		;get value to repeat
	move.b	(a0)+,d5		;get number of times to repeat it
RleRun:
	move.b	d4,(a1)		;store the byte
	add.l		d1,a1			;go down a column
	subq.w	#1,d3			;dec the column counter
	bne.s		ContinueThisColumn2
	move.w	d2,d3
	move.l	a2,a1
	addq.l	#1,a2
ContinueThisColumn2:
	subq.b	#1,d5
	dbeq		d0,RleRun		 ;this loop will always give out because the equality flag
	bra		ContinueThisColumn1;  is set by subtracting 1, from d5, not because the other counter
						 ;  d0 hit -1,  When it does give out d0 will not be decrimented
						 ;  when it should be

