;===========================================================
; RLE picture displayer v1.1
; Decodes a horizontally compressed RLE picture
;
; written by David Phillips <electrum@tfs.net>
; started: 8/19/98
; last update: 11/5/98
;
; small modification by Cullen Sauls: 7/12/03
;
; input: HL = RLE encoded picture, DE = where to display
; output: decoded picture
; destroys: AF, BC, DE, HL
; current size: 32 bytes
; modify IMAGE_SIZE and SENTINEL_VALUE if needed
;===========================================================
IMAGE_SIZE	= 1024
SENTINEL_VALUE	= $91

DispRLE:
 ld bc,IMAGE_SIZE		; we need to copy 
DispRLEL:
 ld a,(hl)			; get the next byte
 cp SENTINEL_VALUE		; is it a run?
 jr z,DispRLERun		; then we need to decode the run
 ldi				; copy the byte, and update counters
DispRLEC:
 ld a,b				; check the low byte and
 or c				; the high byte for 0
 jr nz,DispRLEL			; if not, then we're not done either
 ret				; if it's zero, we're done
DispRLERun:
 inc hl				; move to the run value
 ld a,(hl)			; get the run value
 inc hl				; move to the run count
 push hl			; save source pointer
 ld h,(hl)			; get the run count
 ex de,hl			; swap source and destination pointers
DispRLERunL:
 ld (hl),a			; copy the byte
 inc hl				; increase destination pointer
 dec bc				; decrease byte count
 dec d				; decrease run count
 jr nz,DispRLERunL		; if we're not done, then loop
 ex de,hl			; swap pointers back
 pop hl				; recover source pointer
 inc hl				; advance the source pointer
 jr DispRLEC			; check to see if we should loop
