;===========================================================
; RLE picture displayer v1.0
; Decodes a vertically compressed RLE picture
;
; written by Cullen Sauls <sauls@tamu.edu>
; used David Phillips DispRLE routine as a guideline
; started: 7/12/03
; last update: 7/12/03
;
; input: HL = RLE encoded picture, DE = where to display
; output: decoded picture
; destroys: AF, BC, DE, HL
; current size: 88 bytes
; modify constants if needed
;===========================================================
IMAGE_SIZE      = 1024
BYTES_PER_ROW   = 16
NUM_ROWS        = 64
SENTINEL_VALUE  = $91

DispRLE:
 push hl                            ;save pointers
 push de                            ;"
 ld hl,IMAGE_SIZE                   ;setup checking code
 add hl,de                          ;"
 ld a,h                             ;if HL == 0, then the picture
 or l                               ;is being displayed at the end of the RAM
 jr nz,$+5                          ;in that case
 ld hl,$FFFF                        ;we wont be advancing the row with a calculation
 ld de,DispRLEMod                   ;save the number that marks the end
 ex de,hl                           ;"
 ld (hl),e                          ;"
 inc hl                             ;"
 ld (hl),d                          ;"
 pop de                             ;restore pointers
 pop hl                             ;"
 ld bc,IMAGE_SIZE                   ; we need to copy 
DispRLEL:
 ld a,(hl)                          ; get the next byte
 cp SENTINEL_VALUE                  ; is it a run?
 jr z,DispRLERun                    ; then we need to decode the run
 ldi                                ; copy the byte, and update counters
 dec de                             ;dec DE since "ldi" inc's it
 call DispRLEAdvanceRow             ;advance destination
DispRLEC:
 ld a,b                             ; check the low byte and
 or c                               ; the high byte for 0
 jr nz,DispRLEL                     ; if not, then we're not done either
 ret                                ; if it's zero, we're done
DispRLERun:
 inc hl                             ; move to the run value
 ld a,(hl)                          ; get the run value
 inc hl                             ; move to the run count
 push hl                            ; save source pointer
 ld h,(hl)                          ; get the run count
DispRLERunL:
 ld (de),a                          ; copy the byte
 call DispRLEAdvanceRow             ;advance destination
 dec bc                             ; decrease byte count
 dec h                              ; decrease run count
 jr nz,DispRLERunL                  ; if we're not done, then loop
 pop hl                             ; recover source pointer
 inc hl                             ; advance the source pointer
 jr DispRLEC                        ; check to see if we should loop
DispRLEAdvanceRow:
 push hl                            ;save HL
 ld hl,BYTES_PER_ROW                ;get # bytes per row
 add hl,de                          ;advance to next row
 jr c,DispRLEBackToTop+1            ;check if we need next column (end of RAM case)
DispRLEMod = $+1
 ld de,$0000                        ;check if we went past the memory for the pic
 push hl                            ;(not end of RAM case)
 or a                               ;"
 sbc hl,de                          ;"
 jr z,$+4                           ;" (special check for last byte of RAM)
 jr nc,DispRLEBackToTop             ;"
 pop de                             ;if not new column, restore pointers
 pop hl                             ;"
 ret                                ;"
DispRLEBackToTop:
 pop hl                             ;move destination to top of next column
 ld de,BYTES_PER_ROW * NUM_ROWS     ;"
 or a                               ;"
 sbc hl,de                          ;"
 inc hl                             ;"
 ex de,hl                           ;put destination into DE
 pop hl                             ;restore HL
 ret                                ;done!
